/**
 * @file Board.h
 * @brief a Board osztly interfsze
 */

#ifndef BOARD_H
#define BOARD_H

#include "Shape.h"

class Board  {
	// enum a konstans helyett, az eredmny azonos
    enum { MAXN = 100 };   	
	// Shape-ek trolsa: heterogn kollekci.
	// Pointereket muszj trolni
    Shape* shapes[MAXN];  	
	// A trolt Shape-ek aktulis szma
    unsigned int n;
public:
	// Konstruktor
    Board() : n(0) { }		
    
	// Copy konstruktor
	// Visszavezetjk az rtkad opertorra
	Board(const Board& b) : n(0) {
		*this = b;
	}
	
	// rtkad opertor
	// Ez mr a clone() fggvnyt hvja, deep copy-t kszt
	Board& operator=(const Board& b) {
		if (this == &b)
			return *this;
		clear();
		n = b.n;
		// Minden alakzatot lemsolunk a fajtjnak megfelelen
		for(unsigned int i = 0; i < n; ++i)
			shapes[i] = b.shapes[i]->clone();
		return *this;
	}

	// Mozgats
	// rdekessg: ez lehet const fggvny, mert csak a pointerek
	// ltal mutatott Shape-eket vltoztatja meg. (Azt, hogy a 
	// pointerek hova mutatnak, nem vltoztatja.)
	void move(const Point& p) const
	{
		// Minden alakztatot mozgatunk
		for(unsigned int i = 0; i < n; ++i)
			shapes[i] ->move(p);
	}
	
	// tvesszk egy msik tbla alakzatait
	Board& operator+=(const Board& b);
	
    void put(Shape *sp); 		// j alakzatot tesznk a tblra
    void draw();    			// Minden alakzatot kirajzolunk
    void clear(); 			    // Tbla trlse (felszabadt)
    ~Board() { clear(); }		// Destruktor
};

#endif