/**
 * @file Polygon.h
 * @brief Polygon osztly interfsze
 */

#ifndef POLYGON_H
#define POLYGON_H

#include "Shape.h"

class Polygon : public Shape {
    Point *points;			// dinamikusan trolt pontok
    unsigned int np;		// pontok sszes szma (legalbb 1 van - az sben)
public:
    // Konstruktor
	Polygon(const Point& p0, const Color& c) : Shape(p0, c), np(1)  { 
		points = new Point[np-1]; 
	}

    // Msol
	// Ezt visszavezetjk az operator=-re
	Polygon(const Polygon& p) : Shape(p), points(NULL), np(0) {
		*this = p;
	}
                
    // rtkad opertor
	Polygon& operator=(const Polygon& p) {
		// Fontos ellenrzs
		if (this == &p)
			return *this;
		// Rgi pontokat eldobjuk s jakat foglalunk, majd lemsoljuk ket
		delete [] points;
		p0 = p.p0;
		c = p.c;
		np = p.np;
		points = new Point[np-1];
		for (unsigned int i = 0; i <  np-1; ++i)
			points[i] = p.points[i];
		return *this;
	}
    
	// Pontok szmnak lekrse
	unsigned int getNp() const { return np; }    
    
	// Egy pont lekrse
	Point getP(unsigned int i) const;  
	
	// Pont hozzadsa a Poligonhoz
    void add(const Point& p);            
	
	// Rajzols
    void draw();
	
	// Polygon klnozsa
	Shape* clone() const {
		// A virtulis fggvny segtsgvel itt 
		// helyesen tudunk j Polynomot kszteni
		return new Polygon(*this);
	}
	
	// Destruktor
    ~Polygon() { 
		// Pontok felszabadtsa
		delete[] points; 
	}
};

// Inserter
std::ostream& operator<<(std::ostream&, const Polygon&);

#endif