/**
 *  @file Shape.h
 *  @brief Shape (alakzat) sosztly 
 */

#ifndef SHAPE_H
#define SHAPE_H

#include <iostream>         // ostream
#include "Point.h"
#include "Color.h"

class Shape {
protected:
     Point p0;             // Kezdpont
     Color c;              // Szn
	 // A msol konstruktort elrejtjk.
	 // gy nem hozhat ltre olyan globlis (nem tag-) fggvny, ami szeletelne.
     Shape(const Shape& s) : p0(s.p0), c(s.c) { }
public:
    /// @brief Konstruktor
    /// @param[in] p0 kezdpont
    /// @param[in] c szn
    Shape(const Point& p0, const Color& c) :p0(p0), c(c) {}
    
    /// @brief Kezdpont lekrse
    /// @return kezdpont
    const Point& getP0() const { return p0; }

    /// @brief Szn lekrse
    /// @return color of the shape
    const Color& getC() const { return c; }

    /// @brief Mozgats
    /// @param[in] p ennyivel toljuk el a kezdpontot
    void move(const Point& p);
	
	/// @brief Klnozfggvny, a nem szeletel msolshoz kell
	virtual Shape* clone() const 
	{ 
		// Itt hvjuk az egybknt protected COPY CTOR-t.
		return new Shape(*this); 
	}

	/// @brief Rajzolfggvny
	virtual void draw();

    /// @brief Destruktor
	/// @details Ktelezen virtulis, ha a Shape-bl szrmazni fogunk.
    virtual ~Shape() {}
};

/// Inserter opertor
std::ostream& operator<<(std::ostream&, const Shape&);

#endif // SHAPE_H